//+------------------------------------------------------------------+
//|                                               IncTrixOnArray.mqh |
//|                                                          Integer |
//|                          https://login.mql5.com/ru/users/Integer |
//+------------------------------------------------------------------+
#property copyright "Integer"
#property link      "https://login.mql5.com/ru/users/Integer"
#property version   "1.00"

/*
   External parameters:
   input int            TrixPeriod  =  14;
   input ENUM_MA_METHOD TrixMethod  =  MODE_EMA;

   Declaration:
   #include <IncOnArray/IncTrixOnArray.mqh>
   CTrixOnArray tr;

   In OnInit:
   tr.Init(TrixPeriod,TrixMethod);

   In OnCalculate:
   tr.Solve(rates_total,prev_calculated,data,M1Buffer,M2Buffer,M3Buffer,TBuffer);
*/

#include <IncOnArray/IncMAOnArray.mqh>
//+------------------------------------------------------------------+
//| CTrixOnArray                                                     |
//+------------------------------------------------------------------+
class CTrixOnArray
  {
private:
   int               m_Period;
   ENUM_MA_METHOD    m_Method;
   string            m_Name;
   int               m_VisibleBars;
   int               m_br;
   CMAOnArray        m_1;
   CMAOnArray        m_2;
   CMAOnArray        m_3;
public:
   void Init(int aPeriod=14,ENUM_MA_METHOD aMethod=MODE_EMA)
     {
      m_Period=aPeriod;
      m_Method=aMethod;
      m_3.Init(m_Period,m_Method);
      m_2.Init(m_Period,m_Method);
      m_1.Init(m_Period,m_Method);
      m_Name="TRIX("+IntegerToString(m_Period)+","+m_1.NameMethod()+")";
      m_br=m_1.BarsRequired()*3-3+1;
     }
   void Solve(const int aRatesTotal,
              const int aPrevCalc,
              double  &aData[],
              double  &aM1[],
              double  &aM2[],
              double  &aM3[],
              double  &aTRIX[]
              )
     {
      m_1.Solve(aRatesTotal,aPrevCalc,aData,aM1);
      m_2.Solve(aRatesTotal,aPrevCalc,aM1,aM2);
      m_3.Solve(aRatesTotal,aPrevCalc,aM2,aM3);
      int Start=0;
      if(aPrevCalc==0)
        {
         for(int i=0;i<aRatesTotal;i++)
           {
            if(aM3[i]!=0 && aM3[i]!=EMPTY_VALUE)
              {
               Start=i+1;
               break;
              }
           }
        }
      else
        {
         Start=aPrevCalc-1;
        }
      for(int i=Start;i<aRatesTotal;i++)
        {
         if(aM3[i-1]!=0.0)
           {
            aTRIX[i]=(aM3[i]-aM3[i-1])/aM3[i-1];
           }
         else
           {
            aTRIX[i]=0.0;
           }
        }
     }
   int BarsRequired()
     {
      return(m_br);
     }
   string Name()
     {
      return(m_Name);
     }

   string About()
     {
      return("Integer's TRIXOnArray class. https://login.mql5.com/ru/users/Integer");
     }
  };
//+------------------------------------------------------------------+
